__version__ = '0.1'


class UniqueValue(object):
    """
    Simple, unique object
    """
    __slots__ = ('truthful',)

    def __init__(self, truthful=None):
        self.truthful = truthful

    def __nonzero__(self):
        if self.truthful is None:
            raise TypeError('UniqueValue is not declared as truthful or falsely')
        return self.truthful


"""
Empty-object
"""
empty = UniqueValue(truthful=False)


def coalesce(iterable, ignore=empty, default=empty):
    """
    Return first not empty value from iterable object

    :param iterable - some iterable object
    :param ignore - ignoring value
    :param default - value which returning if all values are `exclude`-values
    :return first not empty value
    """
    return first(lambda x: x != ignore, iterable, default=default)


def first(function, iterable, default=None):
    """
    Return first value from iterable object for which the `function(value)` is truthful

    :param function - some filter-function which returns some logical value
    :param iterable - some iterable object
    :param default - default which returns, when not occurs value with truthful result for `function(value)`
    :return first value from iterable object for which the `function(value)` is truthful
    """
    for value in iterable:
        if function(value):
            return value
    return default



#!/usr/bin/env python

from setuptools import setup, find_packages
from os.path import join, dirname

from coalesce import __version__


setup(
    name='Coalesce',
    version=__version__,
    long_description=open(join(dirname(__file__), 'README.md')).read(),
    description='simple Python library with coalesce function and "magic" empty value',
    author='Alexander Bryazgin',
    author_email='bryazgin64@gmail.com',
    url='https://github.com/bryazginnn/coalesce',
    packages=find_packages(),
)
